@@ -52,7 +52,40 @@
 
 namespace zmq
 {
-class curve_mechanism_base_t : public virtual mechanism_base_t
+class curve_encoding_t
+{
+  public:
+    curve_encoding_t (const char *encode_nonce_prefix_,
+                      const char *decode_nonce_prefix_);
+
+    int encode (msg_t *msg_);
+    int decode (msg_t *msg_, int *error_event_code_);
+
+    uint8_t *get_writable_precom_buffer () { return _cn_precom; }
+    const uint8_t *get_precom_buffer () const { return _cn_precom; }
+
+    typedef uint64_t nonce_t;
+
+    nonce_t get_and_inc_nonce () { return _cn_nonce++; }
+    void set_peer_nonce (nonce_t peer_nonce_) { _cn_peer_nonce = peer_nonce_; };
+
+  private:
+    int check_validity (msg_t *msg_, int *error_event_code_);
+
+    const char *_encode_nonce_prefix;
+    const char *_decode_nonce_prefix;
+
+    nonce_t _cn_nonce;
+    nonce_t _cn_peer_nonce;
+
+    //  Intermediary buffer used to speed up boxing and unboxing.
+    uint8_t _cn_precom[crypto_box_BEFORENMBYTES];
+
+    ZMQ_NON_COPYABLE_NOR_MOVABLE (curve_encoding_t)
+};
+
+class curve_mechanism_base_t : public virtual mechanism_base_t,
+                               public curve_encoding_t
 {
   public:
     curve_mechanism_base_t (session_base_t *session_,
@@ -61,18 +94,8 @@ class curve_mechanism_base_t : public virtual mechanism_base_t
                             const char *decode_nonce_prefix_);
 
     // mechanism implementation
-    virtual int encode (msg_t *msg_);
-    virtual int decode (msg_t *msg_);
-
-  protected:
-    const char *encode_nonce_prefix;
-    const char *decode_nonce_prefix;
-
-    uint64_t cn_nonce;
-    uint64_t cn_peer_nonce;
-
-    //  Intermediary buffer used to speed up boxing and unboxing.
-    uint8_t cn_precom[crypto_box_BEFORENMBYTES];
+    int encode (msg_t *msg_) ZMQ_OVERRIDE;
+    int decode (msg_t *msg_) ZMQ_OVERRIDE;
 };
 }
 
