@@ -36,7 +36,6 @@
 #include "stream_engine_base.hpp"
 #include "ws_address.hpp"
 
-
 #define WS_BUFFER_SIZE 8192
 #define MAX_HEADER_NAME_LENGTH 1024
 #define MAX_HEADER_VALUE_LENGTH 2048
@@ -131,17 +130,27 @@ class ws_engine_t : public stream_engine_base_t
     ws_engine_t (fd_t fd_,
                  const options_t &options_,
                  const endpoint_uri_pair_t &endpoint_uri_pair_,
-                 ws_address_t &address_,
+                 const ws_address_t &address_,
                  bool client_);
     ~ws_engine_t ();
 
   protected:
+    int decode_and_push (msg_t *msg_);
+    int process_command_message (msg_t *msg_);
+    int produce_pong_message (msg_t *msg_);
+    int produce_ping_message (msg_t *msg_);
     bool handshake ();
     void plug_internal ();
+    void start_ws_handshake ();
 
   private:
     int routing_id_msg (msg_t *msg_);
     int process_routing_id_msg (msg_t *msg_);
+    int produce_close_message (msg_t *msg_);
+    int produce_no_msg_after_close (msg_t *msg_);
+    int close_connection_after_close (msg_t *msg_);
+
+    bool select_protocol (const char *protocol);
 
     bool client_handshake ();
     bool server_handshake ();
@@ -164,6 +173,9 @@ class ws_engine_t : public stream_engine_base_t
     char _websocket_protocol[256];
     char _websocket_key[MAX_HEADER_VALUE_LENGTH + 1];
     char _websocket_accept[MAX_HEADER_VALUE_LENGTH + 1];
+
+    int _heartbeat_timeout;
+    msg_t _close_msg;
 };
 }
 
